<?php

namespace Drupal\role_toggle;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Form\FormInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\user\Entity\Role;

class RoleToggleForm implements FormInterface {

  /**
   * { @inheritDoc }
   */
  public function getFormId() {
    return 'role_toggle_form';
  }

  /**
   * Checks access for a specific request.
   *
   * @param \Drupal\Core\Session\AccountInterface $account
   *   Run access checks for this account.
   * @return \Drupal\Core\Access\AccessResultInterface
   *   The access result.
   */
  public function access(AccountInterface $account) {
    return AccessResult::allowedIf(RoleToggle::canToggleAny($account));
  }

  /**
   * { @inheritDoc }
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    foreach (RoleToggle::togglableRoles() as $rid => $role) {
      $enabled = RoleToggle::isEnabledRole($role);
      $class = RoleToggle::indicatorClass($enabled);
      $form['role-toggle']["role-$rid"] = array(
        '#type' => 'submit',
        '#name' => $rid,
        '#value' => $role->label(),
        '#attributes' => array('class' => array($class)),
      );
    }
    if(isset($form['role-toggle'])) {
      $form['role-toggle']['#attached']['library'] = [
        'role_toggle/general',
      ];
    }
    return $form;
  }

  /**
   * { @inheritDoc }
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
  }

  /**
   * { @inheritDoc }
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $triggeringElement = $form_state->getTriggeringElement();
    $rid = $triggeringElement['#name'];
    $role = Role::load($rid);

    // we are paranoid and check access again.
    if (!RoleToggle::userAccess($role)) {
      \Drupal::messenger()->addError(t('Role toggle for role %rid not permitted.', array('%rid' => $role->label())));
      return;
    }

    $hasRole = RoleToggle::toggleRole($role);

    if ($hasRole) {
      \Drupal::messenger()->addStatus(t('Role %role set active.', array('%role' => $role->label())));
    }
    else {
      \Drupal::messenger()->addStatus(t('Role %role set inactive.', array('%role' => $role->label())));
    }
  }

}
